@@ -17,7 +17,7 @@ platform API: that way platform APIs provided by the platform-specific support m
 are correctly composed into the base functionality.
 """
 
-API_VERSION = '1.1_03'
+API_VERSION = '1.1_04'
 
 fdatasync = getattr(os, 'fdatasync', os.fsync)
 
@@ -210,7 +210,16 @@ def getfqdn(name=''):
 # XXX this sometimes requires live internet access for issuing a DNS query in the background.
 hostname = socket.gethostname()
 fqdn = getfqdn(hostname)
-hostid = '%s@%s' % (fqdn, uuid.getnode())
+# some people put the fqdn into /etc/hostname (which is wrong, should be the short hostname)
+# fix this (do the same as "hostname --short" cli command does internally):
+hostname = hostname.split('.')[0]
+
+# uuid.getnode() is problematic in some environments (e.g. OpenVZ, see #3968) where the virtual MAC address
+# is all-zero. uuid.getnode falls back to returning a random value in that case, which is not what we want.
+# thus, we offer BORG_HOST_ID where a user can set an own, unique id for each of his hosts.
+hostid = os.environ.get('BORG_HOST_ID')
+if not hostid:
+    hostid = '%s@%s' % (fqdn, uuid.getnode())
 
 
 def get_process_id():
